@@ -1,6 +1,6 @@
 #
 # Ruby/SDL   Ruby extension library for SDL
-#  Copyright (C) 2001-2005 Ohbayashi Ippei
+#  Copyright (C) 2001-2007 Ohbayashi Ippei
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,8 +22,8 @@ if !defined?(block_given?) then
 end
 
 module SDL
-
-  VERSION = "1.0.0"
+  
+  VERSION = "1.3.1"
 
   class PixelFormat
 
@@ -52,7 +52,6 @@ module SDL
       end
     end
 
-    
     def copyRect(x,y,w,h)
       flagbase=SDL::SWSURFACE|SDL::HWSURFACE|SDL::SRCCOLORKEY|SDL::SRCALPHA
       alpha_flag = self.flags & (SDL::SRCCOLORKEY|SDL::RLEACCEL)
@@ -69,7 +68,11 @@ module SDL
                            self.alpha )
       return new_surface
     end
-
+    
+    def self.blit(src, srcX, srcY, srcW, srcH, dst, dstX, dstY)
+      SDL.blitSurface(src, srcX, srcY, srcW, srcH, dst, dstX, dstY)
+    end
+    
     def self.new(*args)
       case args.size
       when 4
@@ -82,6 +85,10 @@ module SDL
     end
   end
 
+  class Screen
+    def self.open(*args); SDL.setVideoMode(*args) end
+  end
+  
   def color2int(color,format)
     case color
     when Integer
