@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2005 Ohbayashi Ippei
+  Copyright (C) 2001-2007 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -349,7 +349,7 @@ void init_keyEvent()
   rb_define_module_function(mKey,"disableKeyRepeat",sdl_disableKeyRepeat,0);
   rb_define_module_function(mKey,"getKeyName",sdl_getKeyName,1);
 
-  keyState = malloc(SDLK_LAST*sizeof(Uint8));
+  keyState = ALLOC_N(Uint8,SDLK_LAST);
   memset(keyState, SDL_RELEASED, SDLK_LAST);
   
   defineConstForKey();
