@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2005 Ohbayashi Ippei
+  Copyright (C) 2001-2007 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@
 #ifdef HAVE_SDL_MIXER
 
 #include "rubysdl.h"
+#define USE_RWOPS
 #include <SDL_mixer.h>
 
 static int mix_opened=0;
@@ -42,6 +43,14 @@ static void mix_FreeMusic(Mix_Music *music)
   }
 }
 
+static VALUE mix_audioDriverName(VALUE mod)
+{
+  char driver_name[512];
+  if(SDL_AudioDriverName(driver_name, sizeof(driver_name)) == NULL)
+    rb_raise(eSDLError, "No driver has been initialized: %s", SDL_GetError());
+  return rb_str_new2(driver_name);
+}
+
 static VALUE mix_openAudio(VALUE mod,VALUE frequency,VALUE format,
 			   VALUE channels,VALUE chunksize)
 {
@@ -64,7 +73,7 @@ static VALUE mix_querySpec(VALUE mod)
   int channels;
 
   if( !Mix_QuerySpec(&rate,&format,&channels) )
-    rb_raise(eSDLError,"audio have not been opened yet");
+    rb_raise(eSDLError,"audio have not been opened yet: %s", Mix_GetError());
   return rb_ary_new3( 3,INT2NUM(rate),UINT2NUM(format),INT2NUM(channels) );
 }
 
@@ -80,12 +89,12 @@ static VALUE mix_playChannel(VALUE mod,VALUE channel,VALUE wave,VALUE loops)
   int playing_channel;
   
   if( ! rb_obj_is_kind_of(wave,cWave) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Wave is expected");
   Data_Get_Struct(wave,Mix_Chunk,chunk);
   
   playing_channel = Mix_PlayChannel(NUM2INT(channel),chunk,NUM2INT(loops) );
   if( playing_channel == -1 ){
-    rb_raise( eSDLError, "couldn't play wave" );
+    rb_raise( eSDLError, "couldn't play wave: %s", Mix_GetError() );
   }
 
   rb_ary_store(playing_wave,playing_channel,wave);/* to avoid gc problem */
@@ -99,13 +108,13 @@ static VALUE mix_playChannelTimed(VALUE mod,VALUE channel,VALUE wave,VALUE loops
   int playing_channel;
   
   if( ! rb_obj_is_kind_of(wave,cWave) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Wave is expected");
   Data_Get_Struct(wave,Mix_Chunk,chunk);
   
   playing_channel = Mix_PlayChannelTimed(NUM2INT(channel),chunk,NUM2INT(loops),
                                          NUM2INT(ticks));
   if( playing_channel == -1 ){
-    rb_raise( eSDLError, "couldn't play wave" );
+    rb_raise( eSDLError, "couldn't play wave: %s", Mix_GetError() );
   }
 
   rb_ary_store(playing_wave,playing_channel,wave);/* to avoid gc problem */
@@ -119,13 +128,13 @@ static VALUE mix_fadeInChannel(VALUE mod,VALUE channel,VALUE wave,VALUE loops,
   int playing_channel;
   
   if( ! rb_obj_is_kind_of(wave,cWave) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Wave is expected");
   Data_Get_Struct(wave,Mix_Chunk,chunk);
   
   playing_channel = Mix_FadeInChannel(NUM2INT(channel),chunk,NUM2INT(loops),
                                       NUM2INT(ms));
   if( playing_channel == -1 ){
-    rb_raise( eSDLError, "couldn't play wave" );
+    rb_raise( eSDLError, "couldn't play wave: %s", Mix_GetError() );
   }
 
   rb_ary_store(playing_wave,playing_channel,wave);/* to avoid gc problem */
@@ -139,13 +148,13 @@ static VALUE mix_fadeInChannelTimed(VALUE mod,VALUE channel,VALUE wave,VALUE loo
   int playing_channel;
   
   if( ! rb_obj_is_kind_of(wave,cWave) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Wave is expected");
   Data_Get_Struct(wave,Mix_Chunk,chunk);
   
   playing_channel = Mix_FadeInChannelTimed(NUM2INT(channel),chunk,NUM2INT(loops),
                                            NUM2INT(ms), NUM2INT(ticks));
   if( playing_channel == -1 ){
-    rb_raise( eSDLError, "couldn't play wave" );
+    rb_raise( eSDLError, "couldn't play wave: %s", Mix_GetError() );
   }
 
   rb_ary_store(playing_wave,playing_channel,wave);/* to avoid gc problem */
@@ -168,11 +177,22 @@ static VALUE mix_loadWav(VALUE class,VALUE filename)
   wave = Mix_LoadWAV( GETCSTR(filename) );
   if( wave == NULL ){
     rb_raise( eSDLError,"Couldn't load wave file %s: %s",
-	      GETCSTR(filename),SDL_GetError() );
+	      GETCSTR(filename), Mix_GetError() );
   }
   return Data_Wrap_Struct(class,0,mix_FreeChunk,wave);
 }
 
+static VALUE mix_loadWavFromIO(VALUE class, VALUE io)
+{
+  Mix_Chunk *wave;
+  wave = Mix_LoadWAV_RW(rubysdl_RWops_from_ruby_obj(io), 1);
+  if( wave == NULL ){
+    rb_raise(eSDLError,"Couldn't load wave file from IO: %s",
+             Mix_GetError());
+  }
+  return Data_Wrap_Struct(class,0,mix_FreeChunk,wave);
+}
+ 
 /* Volume setting functions and methods : volume in 0..128 */
 static VALUE mix_volume(VALUE mod,VALUE channel,VALUE volume)
 {
@@ -216,7 +236,7 @@ static VALUE mix_expire(VALUE mod, VALUE channel, VALUE ticks)
 static VALUE mix_fading(VALUE mod, VALUE which)
 {
   if( NUM2INT(which) < 0 || Mix_AllocateChannels(-1) <= NUM2INT(which))
-    rb_raise(eSDLError, "channel out of range");
+    rb_raise(eSDLError, "channel %d out of range", NUM2INT(which));
   return INT2FIX(Mix_FadingChannel(which));
 }
   
@@ -232,7 +252,7 @@ static VALUE mix_playMusic(VALUE mod,VALUE music,VALUE loops)
 {
   Mix_Music *mus;
   if( ! rb_obj_is_kind_of(music,cMusic) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Music is expected");
   Data_Get_Struct(music,Mix_Music,mus);
   playing_music=music; /* to avoid gc problem */
   Mix_PlayMusic(mus,NUM2INT(loops));
@@ -243,7 +263,7 @@ static VALUE mix_fadeInMusic(VALUE mod,VALUE music,VALUE loops,VALUE ms)
 {
   Mix_Music *mus;
   if( ! rb_obj_is_kind_of(music,cMusic) )
-    rb_raise(rb_eArgError,"type mismatch");
+    rb_raise(rb_eArgError,"type mismatch: SDL::Mixer::Music is expected");
   Data_Get_Struct(music,Mix_Music,mus);
   Mix_FadeInMusic(mus,NUM2INT(loops),NUM2INT(ms));
   return Qnil;
@@ -290,6 +310,30 @@ static VALUE mix_loadMus(VALUE class,VALUE filename)
 	     "Couldn't load %s: %s",GETCSTR(filename),SDL_GetError());
   return Data_Wrap_Struct(class,0,mix_FreeMusic,music);
 }
+
+#ifdef HAVE_MIX_LOADMUS_RW
+static VALUE mix_loadMusFromString(VALUE class,VALUE str)
+{
+  Mix_Music* music;
+  volatile VALUE result;
+  volatile VALUE buf;
+  
+  StringValue(str);
+  buf = rb_str_dup(str);
+  music = Mix_LoadMUS_RW(SDL_RWFromConstMem(RSTRING(buf)->ptr,
+                                            RSTRING(buf)->len));
+  
+  if( music == NULL )
+    rb_raise(eSDLError,
+	     "Couldn't load from String: %s",Mix_GetError());
+  
+  result = Data_Wrap_Struct(class,0,mix_FreeMusic,music);
+  rb_iv_set(result, "buf", buf);
+  
+  return result;
+}
+#endif
+
 static void defineConstForAudio()
 {
   rb_define_const(mMixer,"FORMAT_U8",UINT2NUM(AUDIO_U8));
@@ -320,6 +364,7 @@ void init_mixer()
   mMixer = rb_define_module_under(mSDL,"Mixer");
   rb_define_module_function(mMixer,"open",mix_openAudio,4);
   rb_define_module_function(mMixer,"spec",mix_querySpec,0);
+  rb_define_module_function(mMixer,"driverName", mix_audioDriverName, 0); 
   rb_define_module_function(mMixer,"playChannel",mix_playChannel,3);
   rb_define_module_function(mMixer,"playChannelTimed", mix_playChannelTimed, 4);
   rb_define_module_function(mMixer,"fadeInChannel", mix_fadeInChannel, 4);
@@ -349,11 +394,15 @@ void init_mixer()
   
   cWave = rb_define_class_under(mMixer,"Wave",rb_cObject);
   rb_define_singleton_method(cWave,"load",mix_loadWav,1);
+  rb_define_singleton_method(cWave,"loadFromIO",mix_loadWavFromIO,1);
   rb_define_method(cWave,"setVolume",mix_wave_volume,1);
 
   cMusic = rb_define_class_under(mMixer,"Music",rb_cObject);
   rb_define_singleton_method(cMusic,"load",mix_loadMus,1);
-
+#ifdef HAVE_MIX_LOADMUS_RW
+  rb_define_singleton_method(cMusic,"loadFromString",mix_loadMusFromString,1);
+#endif
+  
   /* to avoid to do garbage collect when playing */
   playing_wave = rb_ary_new();
   rb_global_variable( &playing_wave );
