@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2005 Ohbayashi Ippei
+  Copyright (C) 2001-2007 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,10 @@
 #ifdef HAVE_SMPEG
 #include "rubysdl.h"
 #include "smpeg/smpeg.h"
+
+#ifdef HAVE_SDL_MIXER
 #include "SDL_mixer.h"
+#endif
 
 static SMPEG_Filter* filters[3];
 #define NULL_FILTER 0
@@ -59,7 +62,7 @@ static VALUE smpeg_load(VALUE class,VALUE filename)
   }
 
   obj = Data_Wrap_Struct(cMPEG,0,SMPEG_delete,mpeg);
-  rb_iv_set(obj,"enable_audio",Qfalse);
+  rb_iv_set(obj,"enable_audio",Qtrue);
   return obj;
 }
 
@@ -82,7 +85,6 @@ static VALUE smpeg_enableAudio(VALUE obj,VALUE enable)
 {
   SMPEG *mpeg;
   Data_Get_Struct(obj,SMPEG,mpeg);
-  SMPEG_enableaudio(mpeg,RTEST(enable));
   rb_iv_set(obj,"enable_audio",enable);
   return Qnil;
 }
@@ -173,6 +175,7 @@ static VALUE smpeg_play(VALUE obj)
   
   Data_Get_Struct(obj,SMPEG,mpeg);
 
+#ifdef HAVE_SDL_MIXER
   use_audio = RTEST(rb_iv_get(obj,"enable_audio")) &&
     Mix_QuerySpec( NULL, NULL, NULL );
 
@@ -194,7 +197,10 @@ static VALUE smpeg_play(VALUE obj)
     Mix_HookMusic(SMPEG_playAudioSDL, mpeg);
     SMPEG_enableaudio(mpeg, 1);
   }
-    
+#else
+  SMPEG_enableaudio(mpeg, RTEST(rb_iv_get(obj,"enable_audio")));
+#endif
+  
   SMPEG_play(mpeg);
   return Qnil;
 }
@@ -212,7 +218,9 @@ static VALUE smpeg_stop(VALUE obj)
   SMPEG *mpeg;
   Data_Get_Struct(obj,SMPEG,mpeg);
   SMPEG_stop(mpeg);
+#ifdef HAVE_SDL_MIXER
   Mix_HookMusic(NULL,NULL);
+#endif
   return Qnil;
 }
 
@@ -248,6 +256,20 @@ static VALUE smpeg_renderFrame(VALUE obj,VALUE framenum)
   return Qnil;
 }
 
+static VALUE smpeg_renderFinal(VALUE obj,VALUE dst, VALUE x, VALUE y)
+{
+  SMPEG *mpeg;
+  SDL_Surface *surface;
+  if( !rb_obj_is_kind_of(dst,cSurface) )
+    rb_raise(rb_eArgError,"type mismatchi(expect Surface)");
+  
+  Data_Get_Struct(obj,SMPEG,mpeg);
+  Data_Get_Struct(dst,SDL_Surface,surface);
+  
+  SMPEG_renderFinal(mpeg, surface, NUM2INT(x), NUM2INT(y));
+  return Qnil;
+}
+  
 static VALUE smpeg_setFilter(VALUE obj,VALUE filter)
 {
   SMPEG *mpeg;
@@ -314,6 +336,7 @@ void init_smpeg()
   rb_define_method(cMPEG,"seek",smpeg_seek,1);
   rb_define_method(cMPEG,"skip",smpeg_skip,1);
   rb_define_method(cMPEG,"renderFrame",smpeg_renderFrame,1);
+  rb_define_method(cMPEG,"renderFinal",smpeg_renderFinal,3);
   rb_define_method(cMPEG,"setFilter",smpeg_setFilter,1);
   
 }
