@@ -1,7 +1,7 @@
 /*
   Ruby/SDL   Ruby extension library for SDL
 
-  Copyright (C) 2001-2005 Ohbayashi Ippei
+  Copyright (C) 2001-2007 Ohbayashi Ippei
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -61,7 +61,9 @@ static VALUE sdl_getVideoSurface(VALUE mod)
 static VALUE sdl_videoDriverName(VALUE mod)
 {
   char namebuf[256];
-  SDL_VideoDriverName( namebuf, sizeof(namebuf) );
+  if(SDL_VideoDriverName(namebuf, sizeof(namebuf)) == NULL) {
+    rb_raise(eSDLError, "SDL is not initialized yet: %s", SDL_GetError());
+  }
   return rb_str_new2( namebuf );
 }
 
@@ -125,6 +127,26 @@ static VALUE sdl_updateRect(VALUE obj,VALUE x,VALUE y,VALUE w,VALUE h)
   return Qnil;
 }
 
+static VALUE sdl_updateRects(int argc, VALUE *argv, VALUE obj)
+{
+  SDL_Surface *screen;
+  SDL_Rect* rects;
+  int i;
+  
+  Data_Get_Struct(obj,SDL_Surface,screen);
+  rects = ALLOCA_N(SDL_Rect, argc);
+
+  for (i=0; i<argc; i++) {
+    rects[i].x = NUM2INT(rb_ary_entry(argv[i], 0));
+    rects[i].y = NUM2INT(rb_ary_entry(argv[i], 1));
+    rects[i].w = NUM2INT(rb_ary_entry(argv[i], 2));
+    rects[i].h = NUM2INT(rb_ary_entry(argv[i], 3));
+  }
+
+  SDL_UpdateRects(screen, argc, rects);
+  return Qnil;
+}
+
 static VALUE sdl_flip(VALUE obj)
 {
   SDL_Surface *screen;
@@ -252,7 +274,7 @@ static VALUE sdl_createSurfaceFrom(VALUE class,VALUE pixels,VALUE w,
   void* pixel_data;
   
   StringValue(pixels);
-  pixel_data = malloc(RSTRING(pixels)->len);
+  pixel_data = ALLOC_N(char, RSTRING(pixels)->len);
   memcpy(pixel_data,RSTRING(pixels)->ptr,RSTRING(pixels)->len);
   
   surface = SDL_CreateRGBSurfaceFrom(pixel_data,NUM2INT(w),NUM2INT(h),
@@ -276,6 +298,18 @@ static VALUE sdl_loadBMP(VALUE class,VALUE filename)
   }
   return Data_Wrap_Struct(class,0,sdl_freeSurface,image);
 }
+
+static VALUE sdl_loadBMPFromIO(VALUE class, VALUE io)
+{
+  volatile VALUE guard = io;
+  SDL_Surface* image;
+  image = SDL_LoadBMP_RW(rubysdl_RWops_from_ruby_obj(io), 1);
+  if(image == NULL)
+    rb_raise(eSDLError, "Couldn't Load BMP file from IO : %s",
+             SDL_GetError());
+  return Data_Wrap_Struct(class,0,sdl_freeSurface,image);
+}
+
 static VALUE sdl_saveBMP(VALUE obj,VALUE filename)
 {
   SDL_Surface *image;
@@ -599,13 +633,74 @@ static VALUE sdl_surface_amask(VALUE obj)
   Data_Get_Struct(obj,SDL_Surface,surface);
   return UINT2NUM(surface->format->Amask);
 }
+static VALUE sdl_surface_rloss(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Rloss);
+}
+static VALUE sdl_surface_gloss(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Gloss);
+}
+static VALUE sdl_surface_bloss(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Bloss);
+}
+static VALUE sdl_surface_aloss(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Aloss);
+}
+static VALUE sdl_surface_rshift(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Rshift);
+}
+static VALUE sdl_surface_gshift(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Gshift);
+}
+static VALUE sdl_surface_bshift(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Bshift);
+}
+static VALUE sdl_surface_ashift(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->Ashift);
+}
+
+static VALUE sdl_surface_pitch(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->pitch);
+}
+static VALUE sdl_surface_bytes_per_pixel(VALUE obj)
+{
+  SDL_Surface *surface;
+  Data_Get_Struct(obj,SDL_Surface,surface);
+  return UINT2NUM(surface->format->BytesPerPixel);
+}
 
 static VALUE sdl_surface_pixels(VALUE obj)
 {
   SDL_Surface *surface;
   Data_Get_Struct(obj,SDL_Surface,surface);
   return rb_str_new(surface->pixels,
-                    surface->w * surface->h * surface->format->BytesPerPixel);
+                    surface->h * surface->pitch);
 }
 
 static void defineConstForVideo()
@@ -621,6 +716,7 @@ static void defineConstForVideo()
   rb_define_const(mSDL,"OPENGL",UINT2NUM(SDL_OPENGL));
   rb_define_const(mSDL,"OPENGLBLIT",UINT2NUM(SDL_OPENGLBLIT));
   rb_define_const(mSDL,"RESIZABLE",UINT2NUM(SDL_RESIZABLE));
+  rb_define_const(mSDL,"NOFRAME", UINT2NUM(SDL_NOFRAME));
   rb_define_const(mSDL,"HWACCEL",UINT2NUM(SDL_HWACCEL));
   rb_define_const(mSDL,"SRCCOLORKEY",UINT2NUM(SDL_SRCCOLORKEY));
   rb_define_const(mSDL,"RLEACCELOK",UINT2NUM(SDL_RLEACCELOK));
@@ -670,6 +766,7 @@ void init_video()
   rb_define_singleton_method(cSurface,"createWithFormat",sdl_createSurfaceWithFormat,8);
   rb_define_singleton_method(cSurface,"new_from",sdl_createSurfaceFrom,9);
   rb_define_singleton_method(cSurface,"loadBMP",sdl_loadBMP,1);
+  rb_define_singleton_method(cSurface,"loadBMPFromIO",sdl_loadBMPFromIO,1);
   rb_define_method(cSurface,"saveBMP",sdl_saveBMP,1);
   rb_define_method(cSurface,"displayFormat",sdl_displayFormat,0);
   rb_define_method(cSurface,"displayFormatAlpha",sdl_displayFormatAlpha,0);
@@ -701,10 +798,21 @@ void init_video()
   rb_define_method(cSurface,"Gmask",sdl_surface_gmask,0);
   rb_define_method(cSurface,"Bmask",sdl_surface_bmask,0);
   rb_define_method(cSurface,"Amask",sdl_surface_amask,0);
+  rb_define_method(cSurface,"Rloss",sdl_surface_rloss,0);
+  rb_define_method(cSurface,"Gloss",sdl_surface_gloss,0);
+  rb_define_method(cSurface,"Bloss",sdl_surface_bloss,0);
+  rb_define_method(cSurface,"Aloss",sdl_surface_aloss,0);
+  rb_define_method(cSurface,"Rshift",sdl_surface_rshift,0);
+  rb_define_method(cSurface,"Gshift",sdl_surface_gshift,0);
+  rb_define_method(cSurface,"Bshift",sdl_surface_bshift,0);
+  rb_define_method(cSurface,"Ashift",sdl_surface_ashift,0);
   rb_define_method(cSurface,"pixels",sdl_surface_pixels,0);
+  rb_define_method(cSurface,"pitch",sdl_surface_pitch,0);
+  rb_define_method(cSurface,"BytesPerPixel",sdl_surface_bytes_per_pixel,0);
   
   cScreen = rb_define_class_under(mSDL,"Screen",cSurface);
   rb_define_method(cScreen,"updateRect",sdl_updateRect,4);
+  rb_define_method(cScreen,"updateRects",sdl_updateRects, -1);
   rb_define_method(cScreen,"flip",sdl_flip,0);
   rb_define_method(cScreen,"toggleFullScreen",sdl_toggleFullScreen,0);
   defineConstForVideo();
