@@ -7,6 +7,11 @@ image = SDL::Surface.loadBMP("icon.bmp")
 image.setColorKey( SDL::SRCCOLORKEY , image[0,0])
 image = image.displayFormat
 
+aimage = SDL::Surface.loadBMP("icon.bmp")
+aimage.setColorKey( SDL::SRCCOLORKEY, aimage[0,0] )
+aimage.setAlpha(SDL::SRCALPHA, 128 )
+aimage = aimage.displayFormat
+
 angle = 0
 
 while true
@@ -28,6 +33,8 @@ while true
 
   SDL.transform(image,screen,angle,1,1,0,0,300,300,0 )
   SDL.transformBlit(image,screen,angle,1,1,0,0,400,200,0 )
+
+  SDL.transformBlit(aimage,screen,angle,1,1,0,0,100,400,0 )
   
   screen.updateRect(0,0,0,0)
 
